* Disclaimer:
* This source code is proprietary of 
* Integrated Signal Processing Systems / RWTH-Aachen / Prof. Meyr (ISS)
* It is part of the DSPstone Benchmarking methodology
* The code, or parts of it may be used for non commercial issues
* All commercial uses require the written consent of  ISS

* file:			irr_one_biquad.asm
* author:		Heiner Loellmann / Andreas Ropers
* date:			18.06.1999
* processor:		TMS320C54x
* project:		dsp-stone
* short description:	benchmarking of an one iir biquad
*                       
*
*	The equations of the filter are:
*       w(n) =    x(n) - a1*w(n-1) - a2*w(n-2)
*       y(n) = b0*w(n) + b1*w(n-1) + b2*w(n-2)
*
*		             w(n)
*   x (n)------(-)---------->-|->----b0---(+)-------> y(n)
*               A             |            A
*               |           |1/z|          |
*               |             | w(n-1)     |
*               |             v            |
*               |-<--a1-----<-|->----b1-->-|
*               |             |            |
*               |           |1/z|          |
*               |             | w(n-2)     |
*               |             v            |
*               |-<--a2-----<--->----b2-->-|
* 
*     The values w(n-1) and w(n-2) are stored in w1 and w2
* 
* ( remark:	 equation after the elimination of w 
*		y(n) = -a1*y(n-1)-a2*y(n-2)+b1*x(n-1)+b2*x(n-2) )
*

	.global startp
	.global prof
	.global endp
	.mmregs

N	.set 4

	.data

x	.word 1			; x(n)

y	.word 1			; y(n)

w	.word 1			; w(n-1)
	.word 2			; w(n-2)

a	.word 1			; a1
	.word 2			; a2

b	.word -1		; b0
	.word -2		; b1
	.word -3		; b2


	.text
startp:	
*	assumption:	SXM=1 FRCT=0 else	
*	SSBX SXM
*	SSBX 0,6
	
	STM #x,AR1
	STM #y,AR2		; not perfomed in the compiler generated code
	STM #w,AR3
	STM #a,AR4
	STM #b,AR5
prof:				; label for the profiling modus
	         ; w(n) = x(n)-a1*w(n-1)-a2*w(n-2)
	LD  *AR3+ , T		; w(n-1) -> TREG
	LD  *AR1  , B		; x(n) -> B
	MAS *AR4+ , B		; B = B-a1*TREG
	LD  *AR3- , T		; w(n-2) -> TREG
	MAS *AR4  , B		; B = B-a2*TREG ( =w(n) )
	
	         ; y(n) = b0*w(n)+b1*w(n-1)+b2*w(n-2)
	STL  B    , *AR2	; B -> y (temporary)
	LD   *AR2 , T		; y -> TREG
	MPY  *AR5+, B		; B = bo*TREG
	LMS  *AR3+, *AR5+	; B = B+b1*w(n-1)
	LMS  *AR3-, *AR5	; B = B+b2*w(n-2)
	
		; updating of w(n-1) and w(n-2) 
	LD   *AR3 , T		; w(n-1) -> TREG
	MVDD *AR2 , *AR3+	; w(n-1)=w(n)	
	ST   T    , *AR3	; w(n-2)=w(n-1)
endp:	nop
	STL  B    , *AR2	; B -> y(n) 
				; not performed in the compiler generated code 
	.end
